@@ -986,6 +986,10 @@ sub draw_form_field_hidden {
 
  
 
+=item checked
+
+1,    .
+
 =back
 
 =cut
@@ -1083,6 +1087,18 @@ sub draw_form_field_password {
 
  
 
+=item value
+
+  (    $data)
+
+=item hidden_value
+
+  (   value)
+
+=item values 
+
+  hidden_value  value.
+
 =back
 
 =cut
